@@ -18,24 +18,98 @@ import fnmatch
 import json
 import logging
 import os
+import pathlib
 import re
 import resource
+import signal
 import sys
 from collections import ChainMap
 from itertools import chain
+from os.path import basename
 from string import Template
 from urllib.parse import parse_qs, urlparse
 
+try:
+    import tomllib  # novermin
+except ModuleNotFoundError:
+    from flux.utils import tomli as tomllib
+
 import flux
 from flux import debugged, job, util
 from flux.constraint.parser import ConstraintParser, ConstraintSyntaxError
 from flux.idset import IDset
-from flux.job import JobspecV1
+from flux.job import JobspecV1, JobWatcher
 from flux.progress import ProgressBar
+from flux.util import dict_merge, set_treedict
 
 LOGGER = logging.getLogger("flux")
 
 
+def decode_signal(val):
+    """
+    Decode a signal as string or number
+    A string can be of the form 'SIGUSR1' or just 'USR1'
+    """
+    if isinstance(val, int):
+        return val
+    try:
+        return int(val)
+    except ValueError:
+        pass  # Fall back to signal name
+    try:
+        return getattr(signal, val)
+    except AttributeError:
+        pass  # Fall back SIG{name}
+    try:
+        return getattr(signal, f"SIG{val}")
+    except AttributeError:
+        pass
+    raise ValueError(f"signal '{val}' is invalid")
+
+
+def decode_duration(val):
+    """
+    Decode a duration as a number or string in FSD
+    """
+    if isinstance(val, (float, int)):
+        return val
+    try:
+        return float(val)
+    except ValueError:
+        pass  # Fall back to fsd
+    return util.parse_fsd(val)
+
+
+def parse_signal_option(arg):
+    """
+    Parse the --signal= option argument of the form SIG@TIME, where
+    both signal and time are optional.
+
+    Returns a dict with signal and timeleft members
+    """
+    signo = signal.SIGUSR1
+    tleft = 60
+    if arg is not None:
+        sig, _, time = arg.partition("@")
+        if time:
+            tleft = time
+        if sig:
+            signo = sig
+    try:
+        signum = decode_signal(signo)
+        if signum <= 0:
+            raise ValueError("signal must be > 0")
+    except ValueError as exc:
+        raise ValueError(f"--signal={arg}: {exc}") from None
+
+    try:
+        timeleft = decode_duration(tleft)
+    except ValueError as exc:
+        raise ValueError(f"--signal={arg}: {exc}") from None
+
+    return {"signum": signum, "timeleft": timeleft}
+
+
 class MiniConstraintParser(ConstraintParser):
     operator_map = {
         None: "properties",
@@ -308,6 +382,122 @@ class EnvFilterAction(argparse.Action):
         setattr(namespace, "env", items)
 
 
+class BatchConfig:
+    """Convenience class for handling a --conf=[FILE|KEY=VAL] option
+
+    Iteratively build a "config" dict from successive updates.
+    """
+
+    loaders = {".toml": tomllib.load, ".json": json.load}
+
+    def __init__(self):
+        self.config = None
+
+    def update_string(self, value):
+        # Update config with JSON or TOML string
+        try:
+            conf = json.loads(value)
+        except json.decoder.JSONDecodeError:
+            # Try TOML
+            try:
+                conf = tomllib.loads(value)
+            except tomllib.TOMLDecodeError:
+                raise ValueError(
+                    "--conf: failed to parse multiline as TOML or JSON"
+                ) from None
+        self.config = dict_merge(self.config, conf)
+        return self
+
+    def update_keyval(self, keyval):
+        # dotted key (e.g. resource.noverify=true)
+        key, _, value = keyval.partition("=")
+        try:
+            value = json.loads(value)
+        except json.decoder.JSONDecodeError:
+            value = str(value)
+        set_treedict(self.config, key, value)
+        return self
+
+    def update_file(self, path, extension=".toml"):
+        # Update from file in the filesystem
+        try:
+            loader = self.loaders[extension]
+        except KeyError:
+            raise ValueError("--conf: {path} must end in .toml or .json")
+        try:
+            with open(path, "rb") as fp:
+                conf = loader(fp)
+        except OSError as exc:
+            raise ValueError(f"--conf: {exc}") from None
+        except (json.decoder.JSONDecodeError, tomllib.TOMLDecodeError) as exc:
+            raise ValueError(f"--conf: parse error: {path}: {exc}") from None
+        self.config = dict_merge(self.config, conf)
+        return self
+
+    def _find_config(self, name):
+        # Find a named config as either TOML or JSON in XDG search path
+        for path in util.xdg_searchpath(subdir="config"):
+            # Take the first matching filename preferring TOML:
+            for ext in (".toml", ".json"):
+                filename = f"{path}/{name}{ext}"
+                if os.path.exists(filename):
+                    return filename, self.loaders[ext]
+        return None, None
+
+    def update_named_config(self, name):
+        # Update from a named configuration file in a standard path or paths.
+        filename, loader = self._find_config(name)
+        if filename is not None:
+            try:
+                with open(filename, "rb") as fp:
+                    self.config = dict_merge(self.config, loader(fp))
+                    return self
+            except (
+                OSError,
+                tomllib.TOMLDecodeError,
+                json.decoder.JSONDecodeError,
+            ) as exc:
+                raise ValueError(f"--conf={name}: {filename}: {exc}") from None
+        raise ValueError(f"--conf: named config '{name}' not found")
+
+    def update(self, value):
+        """
+        Update current config with value using the following rules:
+        - If value contains one or more newlines, parse it as a JSON or
+          TOML string.
+        - Otherwise, if value contains an ``=``, then parse it as a dotted
+          key and value, e.g. ``resource.noverify=true``. The value (part
+          after the ``=``) will be parsed as JSON.
+        - Otherwise, if value ends in ``.toml`` or ``.json`` treat value as
+          a path and attempt to parse contents of file as TOML or JSON.
+        - Otherwise, read a named config from a standard config search path.
+
+        Configuration can be updated iteratively. The end result is available
+        in the ``config`` attribute.
+        """
+        if self.config is None:
+            self.config = {}
+        if "\n" in value:
+            return self.update_string(value)
+        if "=" in value:
+            return self.update_keyval(value)
+        extension = pathlib.Path(value).suffix
+        if extension in (".toml", ".json"):
+            return self.update_file(value, extension)
+        return self.update_named_config(value)
+
+
+class ConfAction(argparse.Action):
+    """Handle batch/alloc --conf option"""
+
+    def __call__(self, parser, namespace, values, option_string=None):
+        conf = getattr(namespace, "conf", None)
+        if conf is None:
+            conf = BatchConfig()
+            setattr(namespace, "conf", conf)
+        conf.update(values)
+
+
 class Xcmd:
     """Represent a Flux job with mutable command and option args"""
 
@@ -541,6 +731,7 @@ class MiniCmd:
         self.flux_handle = None
         self.exitcode = 0
         self.progress = None
+        self.watcher = None
         self.parser = self.create_parser(prog, usage, description, exclude_io)
 
     @staticmethod
@@ -608,6 +799,15 @@ class MiniCmd:
             + "as the value of the attribute. (multiple use OK)",
             metavar="ATTR",
         )
+        parser.add_argument(
+            "--add-file",
+            action="append",
+            help="Add a file at PATH with optional NAME to jobspec. The "
+            + "file will be extracted to {{tmpdir}}/NAME. If NAME is not "
+            + "specified, then the basename of PATH will be used. (multiple "
+            + "use OK)",
+            metavar="[NAME=]PATH",
+        )
         parser.add_argument(
             "--dependency",
             action="append",
@@ -714,11 +914,22 @@ class MiniCmd:
             + "flags:  debug, waitable, novalidate",
             metavar="FLAGS",
         )
+        parser.add_argument(
+            "--signal",
+            help="Schedule delivery of signal SIG at a defined TIME before "
+            + "job expiration. Default SIG is SIGUSR1, default TIME is 60s.",
+            metavar="[SIG][@TIME]",
+        )
         parser.add_argument(
             "--dry-run",
             action="store_true",
             help="Don't actually submit job, just emit jobspec",
         )
+        parser.add_argument(
+            "--quiet",
+            action="store_true",
+            help="Do not print jobid to stdout on submission",
+        )
         parser.add_argument(
             "--debug-emulate", action="store_true", help=argparse.SUPPRESS
         )
@@ -742,6 +953,9 @@ class MiniCmd:
         rlimits = get_filtered_rlimits(args.rlimit)
         if rlimits:
             jobspec.setattr_shell_option("rlimit", rlimits)
+        if args.signal:
+            entry = parse_signal_option(args.signal)
+            jobspec.setattr_shell_option("signal", entry)
 
         # --taskmap is only defined for run/submit, but we check
         # for it in the base jobspec_create() for convenience
@@ -835,6 +1049,20 @@ class MiniCmd:
                 #   due to above conditional)
                 jobspec.setattr(key.lstrip("."), val)
 
+        if args.add_file is not None:
+            for arg in args.add_file:
+                name, _, data = arg.partition("=")
+                if not data:
+                    # No '=' implies path-only argument (no multiline allowed)
+                    if "\n" in name:
+                        raise ValueError("--add-file: file name missing")
+                    data = name
+                    name = basename(data)
+                try:
+                    jobspec.add_file(name, data)
+                except (TypeError, ValueError, OSError) as exc:
+                    raise ValueError(f"--add-file={arg}: {exc}") from None
+
         return jobspec
 
     def submit_async(self, args, jobspec=None):
@@ -1090,6 +1318,8 @@ class SubmitBaseCmd(MiniCmd):
 
     def run_and_exit(self):
         self.flux_handle.reactor_run()
+        if self.watcher:
+            self.exitcode = max(self.watcher.exitcode, self.exitcode)
         sys.exit(self.exitcode)
 
 
@@ -1106,11 +1336,6 @@ class SubmitBulkCmd(SubmitBaseCmd):
         self.t0 = None
 
         super().__init__(prog, usage, description)
-        self.parser.add_argument(
-            "--quiet",
-            action="store_true",
-            help="Do not print jobid to stdout on submission",
-        )
         self.parser.add_argument(
             "--cc",
             metavar="IDSET",
@@ -1165,119 +1390,6 @@ class SubmitBulkCmd(SubmitBaseCmd):
             help="With --progress, show job throughput",
         )
 
-    @staticmethod
-    def output_watch_cb(future, args, jobid, label):
-        """Handle events in the guest.output eventlog"""
-        event = future.get_event()
-        if event and event.name == "data":
-            if "stream" in event.context and "data" in event.context:
-                stream = event.context["stream"]
-                data = event.context["data"]
-                rank = event.context["rank"]
-                if args.label_io:
-                    getattr(args, stream).write(f"{jobid}: {rank}: {data}")
-                else:
-                    getattr(args, stream).write(data)
-
-    def exec_watch_cb(self, future, watch_future, args, jobinfo, label=""):
-        """Handle events in the guest.exec.eventlog"""
-        jobid = jobinfo["id"]
-        event = future.get_event()
-        if event is None:
-            return
-        if args.verbose > 2:
-            ts = event.timestamp - self.t0
-            print(f"{jobid}: {ts:.3f}s exec.{event.name}", file=args.stderr)
-        if args.watch and event and event.name == "shell.init":
-            #  Once the shell.init event is posted, then it is safe to
-            #   begin watching the output eventlog:
-            #
-            job.event_watch_async(
-                self.flux_handle, jobid, eventlog="guest.output"
-            ).then(self.output_watch_cb, args, jobid, label)
-
-            if not args.wait or not args.wait.startswith("exec."):
-                #  Events from this eventlog are no longer needed
-                future.cancel()
-        if args.wait and args.wait == f"exec.{event.name}":
-            # Done with this job: update progress bar if necessary
-            #  and cancel this and the main eventlog futures:
-            #
-            self.progress_update(jobinfo, event=None)
-            future.cancel(stop=True)
-            watch_future.cancel(stop=True)
-
-    @staticmethod
-    def status_to_exitcode(status):
-        """Calculate exitcode from job status"""
-        if os.WIFEXITED(status):
-            status = os.WEXITSTATUS(status)
-        elif os.WIFSIGNALED(status):
-            status = 127 + os.WTERMSIG(status)
-        return status
-
-    def event_watch_cb(self, future, args, jobinfo, label=""):
-        """Handle events in the main job eventlog"""
-        jobid = jobinfo["id"]
-        event = future.get_event()
-        self.progress_update(jobinfo, event=event)
-        if event is None:
-            return
-
-        # Capture first timestamp if not already set
-        if not self.t0 or event.timestamp < self.t0:
-            self.t0 = event.timestamp
-        if args.verbose > 2:
-            ts = event.timestamp - self.t0
-            print(f"{jobid}: {ts:.3f}s {event.name}", file=args.stderr)
-
-        if args.wait and args.wait == event.name:
-            # Done with this job: update progress bar if necessary
-            #  and cancel future
-            #
-            self.progress_update(jobinfo, event=None)
-            future.cancel(stop=True)
-        if event.name == "exception":
-            #
-            #  Handle an exception: update global exitcode and print
-            #   an error:
-            if jobinfo["state"] == "submit":
-                #
-                #  If job was still pending then this job failed
-                #   to execute. Treat it as failure with exitcode = 1
-                #
-                jobinfo["state"] = "failed"
-                if self.exitcode == 0:
-                    self.exitcode = 1
-
-            #  Print a human readable error:
-            exception_type = event.context["type"]
-            note = event.context["note"]
-            print(
-                f"{jobid}: exception: type={exception_type} note={note}",
-                file=args.stderr,
-            )
-        elif event.name == "alloc":
-            jobinfo["state"] = "running"
-        elif event.name == "start" and (args.watch or args.wait.startswith("exec.")):
-            #
-            #  Watch the exec eventlog if the --watch option was provided
-            #   or args.wait starts with 'exec.'
-            #
-            job.event_watch_async(
-                self.flux_handle, jobid, eventlog="guest.exec.eventlog"
-            ).then(self.exec_watch_cb, future, args, jobinfo, label)
-        elif event.name == "finish":
-            #
-            #  Collect exit status and adust self.exitcode if necessary:
-            #
-            jobinfo["state"] = "done"
-            status = self.status_to_exitcode(event.context["status"])
-            if args.verbose:
-                print(f"{jobid}: complete: status={status}", file=args.stderr)
-            if status > self.exitcode:
-                self.exitcode = status
-
     def submit_cb(self, future, args, label=""):
         try:
             jobid = future.get_id()
@@ -1289,41 +1401,56 @@ class SubmitBulkCmd(SubmitBaseCmd):
             self.progress_update(submit_failed=True)
             return
 
-        if args.wait or args.watch:
-            #
-            #  If the user requested to wait for or watch all jobs
-            #   then start watching the main eventlog.
-            #
-            #  Carry along a bit of state for each job so that exceptions
-            #   before the job is running can be handled properly
-            #
-            jobinfo = {"id": jobid, "state": "submit"}
-            fut = job.event_watch_async(self.flux_handle, jobid)
-            fut.then(self.event_watch_cb, args, jobinfo, label)
-            self.progress_update(jobinfo, submit=True)
+        if self.watcher:
+            self.watcher.add_jobid(jobid, args.stdout, args.stderr, args.wait)
         elif self.progress:
             #  Update progress of submission only
             self.progress.update(jps=self.jobs_per_sec())
 
+    def _progress_check(self, args):
+        if args.progress and not self.progress and not sys.stdout.isatty():
+            LOGGER.warning("stdout is not a tty. Ignoring --progress option")
+            args.progress = None
+
+    def watcher_start(self, args):
+
+        if not self.watcher:
+            #  Need to open self.flux_handle if it isn't already in order
+            #  to start the watcher
+            if not self.flux_handle:
+                self.flux_handle = flux.Flux()
+
+            self._progress_check(args)
+
+            self.watcher = JobWatcher(
+                self.flux_handle,
+                progress=args.progress,
+                jps=args.jps,
+                log_events=(args.verbose > 1),
+                log_status=(args.verbose > 0),
+                labelio=args.label_io,
+                wait=args.wait,
+                watch=args.watch,
+            ).start()
+
     def jobs_per_sec(self):
         return (self.progress.count + 1) / self.progress.elapsed
 
     def progress_start(self, args, total):
         """
-        Initialize progress bar if one was requested
+        Initialize job submission progress bar if user requested --progress
+        without --wait or --watch
         """
+        self._progress_check(args)
         if not args.progress or self.progress:
             # progress bar not requested or already started
             return
-        if not sys.stdout.isatty():
-            LOGGER.warning("stdout is not a tty. Ignoring --progress option")
 
-        before = (
-            "PD:{pending:<{width}} R:{running:<{width}} "
-            "CD:{complete:<{width}} F:{fail:<{width}} "
-        )
-        if not (args.wait or args.watch):
-            before = "Submitting {total} jobs: "
+        if args.wait or args.watch:
+            # progress handled in JobWatcher class
+            return
+
+        before = "Submitting {total} jobs: "
         after = "{percent:5.1f}% {elapsed.dt}"
         if args.jps:
             after = "{percent:5.1f}% {jps:4.1f} job/s"
@@ -1334,17 +1461,13 @@ class SubmitBulkCmd(SubmitBaseCmd):
             before=before,
             after=after,
             pending=0,
-            running=0,
-            complete=0,
             fail=0,
             jps=0,
         ).start()
 
-    def progress_update(
-        self, jobinfo=None, submit=False, submit_failed=False, event=None
-    ):
+    def progress_update(self, jobinfo=None, submit=False, submit_failed=False):
         """
-        Update progress bar if one was requested
+        Update submission progress bar if one was requested
         """
         if not self.progress:
             return
@@ -1375,39 +1498,6 @@ class SubmitBulkCmd(SubmitBaseCmd):
                 fail=self.progress.fail + 1,
                 jps=self.jobs_per_sec(),
             )
-        elif event is None:
-            self.progress.update(jps=self.jobs_per_sec())
-        elif event.name == "alloc":
-            self.progress.update(
-                advance=0,
-                pending=self.progress.pending - 1,
-                running=self.progress.running + 1,
-            )
-        elif event.name == "exception" and event.context["severity"] == 0:
-            #
-            #  Exceptions only need to be specially handled in the
-            #   pending state. If the job is running and gets an exception
-            #   then a finish event will be posted and handled below:
-            #
-            if jobinfo["state"] == "submit":
-                self.progress.update(
-                    advance=0,
-                    pending=self.progress.pending - 1,
-                    fail=self.progress.fail + 1,
-                )
-        elif event.name == "finish":
-            if event.context["status"] == 0:
-                self.progress.update(
-                    advance=0,
-                    running=self.progress.running - 1,
-                    complete=self.progress.complete + 1,
-                )
-            else:
-                self.progress.update(
-                    advance=0,
-                    running=self.progress.running - 1,
-                    fail=self.progress.fail + 1,
-                )
 
     @staticmethod
     def cc_list(args):
@@ -1448,12 +1538,15 @@ class SubmitBulkCmd(SubmitBaseCmd):
         args.stdout = sys.stdout
         args.stderr = sys.stderr
 
-        if args.progress:
+        if args.watch or args.wait:
+            self.watcher_start(args)
+
+        elif args.progress:
             self.progress_start(args, len(cclist))
 
         for i in cclist:
             #  substitute any {cc} in args (only if --cc or --bcc):
-            xargs = Xcmd(args, cc=i) if i else args
+            xargs = Xcmd(args, cc=i) if isinstance(i, int) else args
             jobspec = self.jobspec_create(xargs)
 
             #  For now, an idset argument to args.input is not supported
@@ -1496,6 +1589,17 @@ def add_batch_alloc_args(parser):
     Add "batch"-specific resource allocation arguments to parser object
     which deal in slots instead of tasks.
     """
+    parser.add_argument(
+        "--conf",
+        metavar="CONF",
+        default=BatchConfig(),
+        action=ConfAction,
+        help="Set configuration for a child Flux instance. CONF may be a "
+        + "multiline string in JSON or TOML, a configuration key=value, a "
+        + "path to a JSON or TOML file, or a configuration loaded by name "
+        + "from a standard search path. This option may specified multiple "
+        + "times, in which case the config is iteratively updated.",
+    )
     parser.add_argument(
         "--broker-opts",
         metavar="OPTS",
@@ -1543,7 +1647,8 @@ def add_batch_alloc_args(parser):
         help="Distribute allocated resource slots across N individual nodes",
     )
     parser.add_argument(
+        "-x",
         "--exclusive",
         action="store_true",
-        help="With --nodes, allocate nodes exclusively",
+        help="With -N, --nodes, allocate nodes exclusively",
     )
