@@ -15,6 +15,7 @@ import string
 import sys
 import time
 from collections import namedtuple
+from datetime import datetime
 from itertools import chain
 
 import flux.constants
@@ -29,6 +30,13 @@ try:
 except ImportError:
     SchedResourceList = None
 
+# strsignal() is only available in Python 3.8 and up.
+# flux-core's minimum is 3.6.  Use compat library if not available.
+try:
+    from signal import strsignal  # novermin
+except ImportError:
+    from flux.compat36 import strsignal
+
 
 def statetostr(stateid, fmt="L"):
     return raw.flux_job_statetostr(stateid, fmt).decode("utf-8")
@@ -264,6 +272,27 @@ class InfoList(list):
         return ",".join(self)
 
 
+class DependencyList(InfoList):
+    """Post processed list of dependencies."""
+
+    def __init__(self, items):
+        result = []
+        for dep in items:
+            #  Loop through dependencies and handle special cases:
+            if dep.startswith("begin-time="):
+                _, eq, timestamp = dep.partition("=")
+                dt = datetime.fromtimestamp(float(timestamp)).astimezone()
+                if dt.date() == datetime.today().date():
+                    dep = f"begin@{dt:%H:%M}"
+                else:
+                    dep = f"begin@{dt:%b%d-%H:%M}"
+                # Only add seconds if nonzero:
+                if dt.second > 0:
+                    dep += f":{dt:%S}"
+            result.append(dep)
+        super().__init__(result)
+
+
 class JobInfo:
     """
     JobInfo class: encapsulate job-list.list response in an object
@@ -281,7 +310,10 @@ class JobInfo:
         "duration": 0.0,
         "expiration": 0.0,
         "name": "",
+        "cwd": "",
         "queue": "",
+        "project": "",
+        "bank": "",
         "ntasks": "",
         "ncores": "",
         "nnodes": "",
@@ -338,7 +370,7 @@ class JobInfo:
         combined_dict["user"] = combined_dict["annotations"].user
 
         deps = combined_dict.get("dependencies", [])
-        combined_dict["dependencies"] = InfoList(deps)
+        combined_dict["dependencies"] = DependencyList(deps)
 
         #  Set all keys as self._{key} to be found by getattr and
         #   memoized_property decorator:
@@ -374,10 +406,14 @@ class JobInfo:
         return runtime
 
     def get_remaining_time(self):
-        status = str(self.status)
-        if status != "RUN":
+        try:
+            status = str(self.status)
+            if status != "RUN":
+                return 0.0
+            tleft = self.expiration - time.time()
+        except (KeyError, AttributeError):
+            # expiration and/or status attributes may not exist, return 0.0
             return 0.0
-        tleft = self.expiration - time.time()
         if tleft < 0.0:
             return 0.0
         return tleft
@@ -450,6 +486,11 @@ class JobInfo:
         if not isinstance(status, int):
             if self.result_id == flux.constants.FLUX_JOB_RESULT_CANCELED:
                 code = -128
+            elif self.result_id == flux.constants.FLUX_JOB_RESULT_FAILED:
+                #  A job with empty waitstatus could fail if it received a
+                #  fatal exception before starting. Use generic returncode
+                #  of 1.
+                code = 1
         elif os.WIFSIGNALED(status):
             code = -os.WTERMSIG(status)
         elif os.WIFEXITED(status):
@@ -504,7 +545,10 @@ class JobInfo:
         """
         result = {}
         for attr in chain(self.defaults.keys(), self.properties):
-            val = getattr(self, attr)
+            try:
+                val = getattr(self, attr)
+            except AttributeError:
+                val = None
             if val is not None:
                 result[attr] = val
 
@@ -540,6 +584,55 @@ class JobInfo:
 
         return result
 
+    @memoized_property
+    def inactive_reason(self):
+        """
+        Generate contextual exit reason based on how the job ended
+        """
+        state = str(self.state)
+        if state != "INACTIVE":
+            return ""
+        result = str(self.result)
+        if result == "CANCELED":
+            if (
+                self.exception.occurred
+                and self.exception.type == "cancel"
+                and self.exception.note
+            ):
+                return f"Canceled: {self.exception.note}"
+            else:
+                return "Canceled"
+        elif result == "FAILED":
+            # exception.type == "exec" is special case, handled by returncode
+            if (
+                self.exception.occurred
+                and self.exception.type != "exec"
+                and self.exception.severity == 0
+            ):
+                note = None
+                if self.exception.note:
+                    note = f" note={self.exception.note}"
+                return f'Exception: type={self.exception.type}{note or ""}'
+            elif self.returncode > 128:
+                signum = self.returncode - 128
+                try:
+                    sigdesc = strsignal(signum)
+                except ValueError:
+                    sigdesc = f"Signaled {signum}"
+                return sigdesc
+            elif self.returncode == 126:
+                return "Command invoked cannot execute"
+            elif self.returncode == 127:
+                return "command not found"
+            elif self.returncode == 128:
+                return "Invalid argument to exit"
+            else:
+                return f"Exit {self.returncode}"
+        elif result == "TIMEOUT":
+            return "Timeout"
+        else:
+            return f"Exit {self.returncode}"
+
 
 def job_fields_to_attrs(fields):
     # Note there is no attr for "id", it is always returned
@@ -548,6 +641,7 @@ def job_fields_to_attrs(fields):
         "id.dec": (),
         "id.hex": (),
         "id.f58": (),
+        "id.emoji": (),
         "id.kvs": (),
         "id.words": (),
         "id.dothex": (),
@@ -559,7 +653,10 @@ def job_fields_to_attrs(fields):
         "state_single": ("state",),
         "state_emoji": ("state",),
         "name": ("name",),
+        "cwd": ("cwd",),
         "queue": ("queue",),
+        "project": ("project",),
+        "bank": ("bank",),
         "ntasks": ("ntasks",),
         "ncores": ("ncores",),
         "duration": ("duration",),
@@ -591,6 +688,15 @@ def job_fields_to_attrs(fields):
         "dependencies": ("dependencies",),
         "contextual_info": ("state", "dependencies", "annotations", "nodelist"),
         "contextual_time": ("state", "t_run", "t_cleanup", "duration"),
+        "inactive_reason": (
+            "state",
+            "result",
+            "waitstatus",
+            "exception_occurred",
+            "exception_severity",
+            "exception_type",
+            "exception_note",
+        ),
         # Special cases, pointers to sub-dicts in annotations
         "sched": ("annotations",),
         "user": ("annotations",),
@@ -630,6 +736,7 @@ class JobInfoFormat(flux.util.OutputFormat):
         "id.dec": "JOBID",
         "id.hex": "JOBID",
         "id.f58": "JOBID",
+        "id.emoji": "JOBID",
         "id.kvs": "JOBID",
         "id.words": "JOBID",
         "id.dothex": "JOBID",
@@ -641,7 +748,10 @@ class JobInfoFormat(flux.util.OutputFormat):
         "state_single": "S",
         "state_emoji": "STATE",
         "name": "NAME",
+        "cwd": "CWD",
         "queue": "QUEUE",
+        "project": "PROJECT",
+        "bank": "BANK",
         "ntasks": "NTASKS",
         "ncores": "NCORES",
         "duration": "DURATION",
@@ -673,6 +783,7 @@ class JobInfoFormat(flux.util.OutputFormat):
         "dependencies": "DEPENDENCIES",
         "contextual_info": "INFO",
         "contextual_time": "TIME",
+        "inactive_reason": "INACTIVE-REASON",
         # The following are special pre-defined cases per RFC27
         "annotations.sched.t_estimate": "T_ESTIMATE",
         "annotations.sched.reason_pending": "REASON",
