@@ -9,6 +9,7 @@
 ###############################################################
 
 import argparse
+import base64
 import copy
 import errno
 import glob
@@ -19,6 +20,7 @@ import os
 import re
 import shutil
 import signal
+import stat
 import sys
 import threading
 import traceback
@@ -30,8 +32,10 @@ from typing import Mapping
 
 import yaml
 
+# tomllib added to standard library in Python 3.11
+# flux-core minimum is Python 3.6.
 try:
-    import tomllib
+    import tomllib  # novermin
 except ModuleNotFoundError:
     from flux.utils import tomli as tomllib
 
@@ -199,6 +203,65 @@ class TreedictAction(argparse.Action):
             setattr(namespace, self.dest, result)
 
 
+class FilterAction(argparse.Action):
+    def __call__(self, parser, namespace, values, option_string=None):
+        setattr(namespace, self.dest, values)
+        setattr(namespace, "filtered", True)
+
+
+class FilterActionSetUpdate(argparse.Action):
+    def __call__(self, parser, namespace, values, option_string=None):
+        setattr(namespace, "filtered", True)
+        values = values.split(",")
+        getattr(namespace, self.dest).update(values)
+
+
+# pylint: disable=redefined-builtin
+class FilterTrueAction(argparse.Action):
+    def __init__(
+        self,
+        option_strings,
+        dest,
+        const=True,
+        default=False,
+        required=False,
+        help=None,
+        metavar=None,
+    ):
+        super(FilterTrueAction, self).__init__(
+            option_strings=option_strings,
+            dest=dest,
+            nargs=0,
+            const=const,
+            default=default,
+            help=help,
+        )
+
+    def __call__(self, parser, namespace, values, option_string=None):
+        setattr(namespace, self.dest, self.const)
+        setattr(namespace, "filtered", True)
+
+
+class YesNoAction(argparse.Action):
+    """Simple argparse.Action for options with yes|no arguments"""
+
+    def __init__(
+        self,
+        option_strings,
+        dest,
+        help=None,
+        metavar="[yes|no]",
+    ):
+        super().__init__(
+            option_strings=option_strings, dest=dest, help=help, metavar=metavar
+        )
+
+    def __call__(self, parser, namespace, value, option_string=None):
+        if value not in ["yes", "no"]:
+            raise ValueError(f"{option_string} requires either 'yes' or 'no'")
+        setattr(namespace, self.dest, value == "yes")
+
+
 class CLIMain(object):
     def __init__(self, logger=None):
         if logger is None:
@@ -283,6 +346,9 @@ def parse_datetime(string, now=None):
     if now is None:
         now = datetime.now().astimezone()
 
+    if string == "now":
+        return now
+
     if string.startswith("+"):
         timestamp = now.timestamp() + parse_fsd(string[1:])
         return datetime.fromtimestamp(timestamp).astimezone()
@@ -312,16 +378,28 @@ class UtilDatetime(datetime):
     def __format__(self, fmt):
         # The string "::" is used to split the strftime() format from
         # any Python format spec:
-        vals = fmt.split("::", 1)
+        timefmt, *spec = fmt.split("::", 1)
 
-        # Call strftime() to get the formatted datetime as a string
-        result = self.strftime(vals[0])
+        if self == datetime.fromtimestamp(0.0):
+            result = ""
+        else:
+            # Call strftime() to get the formatted datetime as a string
+            result = self.strftime(timefmt or "%FT%T")
+
+        spec = spec[0] if spec else ""
+
+        # Handling of the 'h' suffix on spec is required here, since the
+        # UtilDatetime format() is called _after_ UtilFormatter.format_field()
+        # (where this option is handled for other types)
+        if spec.endswith("h"):
+            if not result:
+                result = "-"
+            spec = spec[:-1] + "s"
 
         # If there was a format spec, apply it here:
-        try:
-            return f"{{0:{vals[1]}}}".format(result)
-        except IndexError:
-            return result
+        if spec:
+            return f"{{0:{spec}}}".format(result)
+        return result
 
 
 def fsd(secs):
@@ -346,6 +424,12 @@ def fsd(secs):
     return strtmp
 
 
+def empty_outputs():
+    localepoch = datetime.fromtimestamp(0.0).strftime("%FT%T")
+    empty = ("", "0s", "0.0", "0:00:00", "1970-01-01T00:00:00", localepoch)
+    return empty
+
+
 class UtilFormatter(Formatter):
     # pylint: disable=too-many-branches
 
@@ -421,9 +505,11 @@ class UtilFormatter(Formatter):
             denote_truncation = True
             spec = spec[:-1]
 
-        if spec.endswith("h"):
-            localepoch = datetime.fromtimestamp(0.0).strftime("%FT%T")
-            basecases = ("", "0s", "0.0", "0:00:00", "1970-01-01T00:00:00", localepoch)
+        # Note: handling of the 'h' suffix for UtilDatetime objects
+        # must be deferred to the UtilDatetetime format() method, since
+        # that method will be called after this one:
+        if spec.endswith("h") and not isinstance(value, UtilDatetime):
+            basecases = empty_outputs()
             value = "-" if str(value) in basecases else str(value)
             spec = spec[:-1] + "s"
         retval = super().format_field(value, spec)
@@ -622,7 +708,7 @@ class OutputFormat:
 
         #  Iterate over all items, rebuilding lst each time to contain
         #  only those fields that resulted in non-"empty" strings:
-        empty = ("", "0", "0s", "0.0", "0:00:00", "1970-01-01T00:00:00")
+        empty = empty_outputs()
         for item in items:
             lst = [x for x in lst if formatter.format(x["fmt"], item) in empty]
 
@@ -674,11 +760,11 @@ class OutputFormat:
         if not no_header:
             print(formatter.header())
         for item in items:
-            if callable(pre):
-                pre(item)
             line = formatter.format(item)
             if not line:
                 continue
+            if callable(pre):
+                pre(item)
             try:
                 print(line)
             except UnicodeEncodeError:
@@ -963,6 +1049,23 @@ def dict_merge(src, new):
     return src
 
 
+def xdg_searchpath(subdir=""):
+    """
+    Build standard Flux config search path based on XDG specification
+    """
+    #  Start with XDG_CONFIG_HOME (or ~/.config) since it is the
+    #  highest precedence:
+    confdirs = [os.getenv("XDG_CONFIG_HOME") or f"{Path.home()}/.config"]
+
+    #  Append XDG_CONFIG_DIRS as colon separated path (or /etc/xdg)
+    #  Note: colon separated paths are in order of precedence.
+    confdirs += (os.getenv("XDG_CONFIG_DIRS") or "/etc/xdg").split(":")
+
+    #  Append "/flux" (with optional subdir) to all members of
+    #  confdirs to build searchpath:
+    return [Path(directory, "flux", subdir) for directory in confdirs]
+
+
 class UtilConfig:
     """
     Very simple class for loading hierarchical configuration for Flux
@@ -1008,17 +1111,7 @@ class UtilConfig:
         #  specification. Later this will be reversed since we want
         #  to traverse the paths in _reverse_ precedence order in this
         #  implementation.
-        #
-        #  Start with XDG_CONFIG_HOME (or ~/.config) since it is the
-        #  highest precedence:
-        confdirs = [os.getenv("XDG_CONFIG_HOME") or f"{Path.home()}/.config"]
-
-        #  Append XDG_CONFIG_DIRS as colon separated path (or /etc/xdg)
-        #  Note: colon separated paths are in order of precedence.
-        confdirs += (os.getenv("XDG_CONFIG_DIRS") or "/etc/xdg").split(":")
-
-        #  Append "/flux" to all members of confdirs to build searchpath:
-        self.searchpath = [Path(directory, "flux") for directory in confdirs]
+        self.searchpath = xdg_searchpath()
 
         #  Reorder searchpath into reverse precedence order
         self.searchpath.reverse()
@@ -1042,7 +1135,7 @@ class UtilConfig:
                     continue
 
                 try:
-                    with open(filepath) as ofile:
+                    with open(filepath, "rb") as ofile:
                         conf = self.extension_handlers[ppath.suffix](ofile)
                 except (
                     tomllib.TOMLDecodeError,
@@ -1176,3 +1269,47 @@ class UtilConfig:
 
     def __getattr__(self, attr):
         return self.config[attr]
+
+
+class Fileref(dict):
+    """
+    Construct a RFC 37 data file object from a data parameter and optional
+    permissions and encoding. ``Fileref`` is a subclass of ``dict`` so it
+    may be used in place of a dict and is directly serializable to JSON.
+
+    If ``data`` is a dict, then a file object with no encoding is created
+    and the dict is stored in ``data`` for eventual encoding as JSON
+    content. Otherwise, if ``encoding`` is set, then data is presumed to
+    already be encoded and is stored verbatim.  If data is not a dict,
+    and encoding is not set, then data is assumed to be the path to file
+    in the filesystem, in which case the encoding will be 'utf-8' if the
+    file contents can be encoded as such, otherwise 'base64'.
+
+    Args:
+        data (dict, str, bytes): File data or path as explained above.
+        perms: File permissions (default 0600 octal)
+        encoding: Explicit encoding if `data` is not a dict or filename.
+    """
+
+    def __init__(self, data, perms=0o0600, encoding=None):
+        ref = {"mode": perms | stat.S_IFREG}
+        if isinstance(data, dict):
+            ref["data"] = data
+        elif encoding is not None:
+            if encoding not in ("base64", "utf-8"):
+                raise ValueError("invalid encoding: {encoding}")
+            ref["data"] = data
+            ref["encoding"] = encoding
+        else:
+            st = os.stat(data)
+            ref["size"] = st.st_size
+            ref["mode"] = st.st_mode
+            with open(data, "rb") as infp:
+                try:
+                    ref["data"] = infp.read().decode("utf-8")
+                    ref["encoding"] = "utf-8"
+                except UnicodeError:
+                    infp.seek(0)
+                    ref["data"] = base64.b64encode(infp.read()).decode("utf-8")
+                    ref["encoding"] = "base64"
+        super().__init__(**ref)
