@@ -1084,9 +1084,7 @@ uint32_t flux_rpc_get_matchtag (flux_future_t *f);
 
 
 uint32_t flux_rpc_get_nodeid (flux_future_t *f);
-# 1694 "/__w/flux-python/flux-python/src/_core_clean.h"
-int flux_panic (flux_t *h, uint32_t nodeid, int flags, const char *reason);
-# 1723 "/__w/flux-python/flux-python/src/_core_clean.h"
+# 1690 "/__w/flux-python/flux-python/src/_core_clean.h"
 enum event_flags {
     FLUX_EVENT_PRIVATE = 1,
 };
@@ -1167,7 +1165,7 @@ flux_future_t *flux_event_publish_raw (flux_t *h,
 
 
 int flux_event_publish_get_seq (flux_future_t *f, int *seq);
-# 1834 "/__w/flux-python/flux-python/src/_core_clean.h"
+# 1801 "/__w/flux-python/flux-python/src/_core_clean.h"
 enum {
     FLUX_MODSTATE_INIT = 0,
     FLUX_MODSTATE_RUNNING = 1,
@@ -1193,7 +1191,7 @@ bool flux_module_debug_test (flux_t *h, int flag, bool clear);
 
 
 int flux_module_set_running (flux_t *h);
-# 1907 "/__w/flux-python/flux-python/src/_core_clean.h"
+# 1874 "/__w/flux-python/flux-python/src/_core_clean.h"
 const char *flux_attr_get (flux_t *h, const char *name);
 
 
@@ -1204,12 +1202,16 @@ const char *flux_attr_get (flux_t *h, const char *name);
 int flux_attr_set (flux_t *h, const char *name, const char *val);
 
 
-
 int flux_attr_set_cacheonly (flux_t *h, const char *name, const char *val);
 
 
 
 
+const char *flux_attr_cache_first (flux_t *h);
+const char *flux_attr_cache_next (flux_t *h);
+
+
+
 
 int flux_get_rank (flux_t *h, uint32_t *rank);
 
@@ -1228,7 +1230,7 @@ const char *flux_get_hostbyrank (flux_t *h, uint32_t rank);
 
 
 int flux_get_rankbyhost (flux_t *h, const char *host);
-# 1955 "/__w/flux-python/flux-python/src/_core_clean.h"
+# 1926 "/__w/flux-python/flux-python/src/_core_clean.h"
 char *flux_hostmap_lookup (flux_t *h,
                            const char *targets,
                            flux_error_t *errp);
@@ -1239,7 +1241,7 @@ char *flux_hostmap_lookup (flux_t *h,
 
 
 int flux_get_instance_starttime (flux_t *h, double *starttime);
-# 1993 "/__w/flux-python/flux-python/src/_core_clean.h"
+# 1964 "/__w/flux-python/flux-python/src/_core_clean.h"
 enum flux_conf_flags {
     FLUX_CONF_INSTALLED=0,
     FLUX_CONF_INTREE=1,
@@ -1292,13 +1294,13 @@ int flux_conf_vunpack (const flux_conf_t *conf,
 int flux_conf_unpack (const flux_conf_t *conf,
                       flux_error_t *error,
                       const char *fmt, ...);
-# 2077 "/__w/flux-python/flux-python/src/_core_clean.h"
+# 2048 "/__w/flux-python/flux-python/src/_core_clean.h"
 flux_future_t *flux_barrier (flux_t *h, const char *name, int nprocs);
-# 2120 "/__w/flux-python/flux-python/src/_core_clean.h"
+# 2091 "/__w/flux-python/flux-python/src/_core_clean.h"
 flux_future_t *flux_service_register (flux_t *h, const char *name);
-# 2132 "/__w/flux-python/flux-python/src/_core_clean.h"
+# 2103 "/__w/flux-python/flux-python/src/_core_clean.h"
 flux_future_t *flux_service_unregister (flux_t *h, const char *name);
-# 2184 "/__w/flux-python/flux-python/src/_core_clean.h"
+# 2155 "/__w/flux-python/flux-python/src/_core_clean.h"
 const char *flux_core_version_string (void);
 
 
@@ -1307,7 +1309,7 @@ const char *flux_core_version_string (void);
 
 
 int flux_core_version (int *major, int *minor, int *patch);
-# 2222 "/__w/flux-python/flux-python/src/_core_clean.h"
+# 2193 "/__w/flux-python/flux-python/src/_core_clean.h"
 enum {
     FLUX_PLUGIN_RTLD_LAZY = 1,
     FLUX_PLUGIN_RTLD_NOW = 2,
@@ -1385,7 +1387,7 @@ flux_plugin_f flux_plugin_match_handler (flux_plugin_t *p, const char *topic);
 int flux_plugin_register (flux_plugin_t *p,
                           const char *name,
                           const struct flux_plugin_handler t[]);
-# 2309 "/__w/flux-python/flux-python/src/_core_clean.h"
+# 2280 "/__w/flux-python/flux-python/src/_core_clean.h"
 int flux_plugin_aux_set (flux_plugin_t *p,
                          const char *key,
                          void *val,
@@ -1447,14 +1449,14 @@ int flux_plugin_arg_unpack (flux_plugin_arg_t *args, int flags,
                             const char *fmt, ...);
 int flux_plugin_arg_vunpack (flux_plugin_arg_t *args, int flags,
                              const char *fmt, va_list ap);
-# 2378 "/__w/flux-python/flux-python/src/_core_clean.h"
+# 2349 "/__w/flux-python/flux-python/src/_core_clean.h"
 int flux_plugin_call (flux_plugin_t *p, const char *name,
                       flux_plugin_arg_t *args);
-# 2388 "/__w/flux-python/flux-python/src/_core_clean.h"
+# 2359 "/__w/flux-python/flux-python/src/_core_clean.h"
 int flux_plugin_load_dso (flux_plugin_t *p, const char *path);
-# 2421 "/__w/flux-python/flux-python/src/_core_clean.h"
+# 2392 "/__w/flux-python/flux-python/src/_core_clean.h"
 flux_future_t *flux_sync_create (flux_t *h, double minimum);
-# 2453 "/__w/flux-python/flux-python/src/_core_clean.h"
+# 2424 "/__w/flux-python/flux-python/src/_core_clean.h"
 bool flux_disconnect_match (const flux_msg_t *msg1, const flux_msg_t *msg2);
 
 
@@ -1474,7 +1476,7 @@ bool flux_cancel_match (const flux_msg_t *msg1, const flux_msg_t *msg2);
 int flux_msglist_cancel (flux_t *h,
                          struct flux_msglist *l,
                          const flux_msg_t *msg);
-# 2522 "/__w/flux-python/flux-python/src/_core_clean.h"
+# 2493 "/__w/flux-python/flux-python/src/_core_clean.h"
 void flux_stats_count (flux_t *h, const char *name, ssize_t count);
 
 
@@ -1510,10 +1512,10 @@ void flux_stats_set_prefix (flux_t *h, const char *fmt, ...);
 
 
 bool flux_stats_enabled (flux_t *h, const char *metric);
-# 2605 "/__w/flux-python/flux-python/src/_core_clean.h"
+# 2576 "/__w/flux-python/flux-python/src/_core_clean.h"
 typedef struct flux_kvsdir flux_kvsdir_t;
 typedef struct flux_kvsitr flux_kvsitr_t;
-# 2620 "/__w/flux-python/flux-python/src/_core_clean.h"
+# 2591 "/__w/flux-python/flux-python/src/_core_clean.h"
 flux_kvsdir_t *flux_kvsdir_create (flux_t *handle, const char *rootref,
                                    const char *key, const char *json_str);
 void flux_kvsdir_destroy (flux_kvsdir_t *dir);
@@ -1559,7 +1561,7 @@ const char *flux_kvsdir_rootref (const flux_kvsdir_t *dir);
 
 
 char *flux_kvsdir_key_at (const flux_kvsdir_t *dir, const char *key);
-# 2693 "/__w/flux-python/flux-python/src/_core_clean.h"
+# 2664 "/__w/flux-python/flux-python/src/_core_clean.h"
 flux_future_t *flux_kvs_lookup (flux_t *h, const char *ns, int flags,
                                 const char *key);
 flux_future_t *flux_kvs_lookupat (flux_t *h, int flags, const char *key,
@@ -1582,14 +1584,14 @@ const char *flux_kvs_lookup_get_key (flux_future_t *f);
 
 
 int flux_kvs_lookup_cancel (flux_future_t *f);
-# 2743 "/__w/flux-python/flux-python/src/_core_clean.h"
+# 2714 "/__w/flux-python/flux-python/src/_core_clean.h"
 flux_future_t *flux_kvs_getroot (flux_t *h, const char *ns, int flags);
-# 2752 "/__w/flux-python/flux-python/src/_core_clean.h"
+# 2723 "/__w/flux-python/flux-python/src/_core_clean.h"
 int flux_kvs_getroot_get_treeobj (flux_future_t *f, const char **treeobj);
 int flux_kvs_getroot_get_blobref (flux_future_t *f, const char **blobref);
 int flux_kvs_getroot_get_sequence (flux_future_t *f, int *seq);
 int flux_kvs_getroot_get_owner (flux_future_t *f, uint32_t *owner);
-# 2784 "/__w/flux-python/flux-python/src/_core_clean.h"
+# 2755 "/__w/flux-python/flux-python/src/_core_clean.h"
 typedef struct flux_kvs_txn flux_kvs_txn_t;
 
 flux_kvs_txn_t *flux_kvs_txn_create (void);
@@ -1619,7 +1621,7 @@ int flux_kvs_txn_unlink (flux_kvs_txn_t *txn, int flags,
 int flux_kvs_txn_symlink (flux_kvs_txn_t *txn, int flags,
                           const char *key, const char *ns,
                           const char *target);
-# 2862 "/__w/flux-python/flux-python/src/_core_clean.h"
+# 2833 "/__w/flux-python/flux-python/src/_core_clean.h"
 enum kvs_commit_flags {
     FLUX_KVS_NO_MERGE = 1,
     FLUX_KVS_TXN_COMPACT = 2,
@@ -1637,21 +1639,21 @@ flux_future_t *flux_kvs_fence (flux_t *h, const char *ns, int flags,
 int flux_kvs_commit_get_treeobj (flux_future_t *f, const char **treeobj);
 int flux_kvs_commit_get_rootref (flux_future_t *f, const char **rootref);
 int flux_kvs_commit_get_sequence (flux_future_t *f, int *rootseq);
-# 2918 "/__w/flux-python/flux-python/src/_core_clean.h"
+# 2889 "/__w/flux-python/flux-python/src/_core_clean.h"
 flux_future_t *flux_kvs_copy (flux_t *h,
                               const char *srcns,
                               const char *srckey,
                               const char *dstns,
                               const char *dstkey,
                               int commit_flags);
-# 2935 "/__w/flux-python/flux-python/src/_core_clean.h"
+# 2906 "/__w/flux-python/flux-python/src/_core_clean.h"
 flux_future_t *flux_kvs_move (flux_t *h,
                               const char *srcns,
                               const char *srckey,
                               const char *dstns,
                               const char *dstkey,
                               int commit_flags);
-# 2958 "/__w/flux-python/flux-python/src/_core_clean.h"
+# 2929 "/__w/flux-python/flux-python/src/_core_clean.h"
 enum kvs_op {
     FLUX_KVS_READDIR = 1,
     FLUX_KVS_READLINK = 2,
@@ -1663,7 +1665,7 @@ enum kvs_op {
     FLUX_KVS_WATCH_UNIQ = 128,
     FLUX_KVS_WATCH_APPEND = 256
 };
-# 2979 "/__w/flux-python/flux-python/src/_core_clean.h"
+# 2950 "/__w/flux-python/flux-python/src/_core_clean.h"
 flux_future_t *flux_kvs_namespace_create (flux_t *h, const char *ns,
                                           uint32_t owner, int flags);
 flux_future_t *flux_kvs_namespace_create_with (flux_t *h, const char *ns,
@@ -1682,9 +1684,8 @@ int flux_kvs_wait_version (flux_t *h, const char *ns, int version);
 
 
 
-
 int flux_kvs_dropcache (flux_t *h);
-# 3027 "/__w/flux-python/flux-python/src/_core_clean.h"
+# 2997 "/__w/flux-python/flux-python/src/_core_clean.h"
 enum job_submit_flags {
     FLUX_JOB_PRE_SIGNED = 1,
     FLUX_JOB_DEBUG = 2,
@@ -1692,6 +1693,10 @@ enum job_submit_flags {
     FLUX_JOB_NOVALIDATE = 8,
 };
 
+enum job_event_watch_flags {
+    FLUX_JOB_EVENT_WATCH_WAITCREATE = 1,
+};
+
 enum job_urgency {
     FLUX_JOB_URGENCY_MIN = 0,
     FLUX_JOB_URGENCY_HOLD = FLUX_JOB_URGENCY_MIN,
@@ -1746,10 +1751,10 @@ typedef uint64_t flux_jobid_t;
 
 
 int flux_job_id_parse (const char *s, flux_jobid_t *id);
-# 3097 "/__w/flux-python/flux-python/src/_core_clean.h"
+# 3071 "/__w/flux-python/flux-python/src/_core_clean.h"
 int flux_job_id_encode (flux_jobid_t id, const char *type,
                         char *buf, size_t bufsz);
-# 3108 "/__w/flux-python/flux-python/src/_core_clean.h"
+# 3082 "/__w/flux-python/flux-python/src/_core_clean.h"
 const char *flux_job_statetostr (flux_job_state_t state, const char *fmt);
 
 
@@ -1863,16 +1868,16 @@ flux_future_t *flux_job_result (flux_t *h, flux_jobid_t id, int flags);
 
 int flux_job_result_get (flux_future_t *f,
                          const char **json_str);
-# 3239 "/__w/flux-python/flux-python/src/_core_clean.h"
+# 3213 "/__w/flux-python/flux-python/src/_core_clean.h"
 int flux_job_result_get_unpack (flux_future_t *f, const char *fmt, ...);
-# 3253 "/__w/flux-python/flux-python/src/_core_clean.h"
+# 3227 "/__w/flux-python/flux-python/src/_core_clean.h"
 int flux_job_timeleft (flux_t *h, flux_error_t *errp, double *timeleft);
-# 3269 "/__w/flux-python/flux-python/src/_core_clean.h"
+# 3243 "/__w/flux-python/flux-python/src/_core_clean.h"
 char *flux_unwrap_string (const char *in,
                           bool verify,
                           uint32_t *userid,
                           flux_error_t *error);
-# 3297 "/__w/flux-python/flux-python/src/_core_clean.h"
+# 3271 "/__w/flux-python/flux-python/src/_core_clean.h"
 typedef struct flux_jobspec1 flux_jobspec1_t;
 typedef flux_error_t flux_jobspec1_error_t;
 
@@ -1970,7 +1975,7 @@ char *flux_jobspec1_encode (flux_jobspec1_t *jobspec, size_t flags);
 
 flux_jobspec1_t *flux_jobspec1_decode (const char *s,
                                        flux_jobspec1_error_t *error);
-# 3408 "/__w/flux-python/flux-python/src/_core_clean.h"
+# 3382 "/__w/flux-python/flux-python/src/_core_clean.h"
 flux_jobspec1_t *flux_jobspec1_from_command (int argc,
                                              char **argv,
                                              char **env,
@@ -1982,7 +1987,7 @@ flux_jobspec1_t *flux_jobspec1_from_command (int argc,
 
 
 void flux_jobspec1_destroy (flux_jobspec1_t *jobspec);
-# 3450 "/__w/flux-python/flux-python/src/_core_clean.h"
+# 3424 "/__w/flux-python/flux-python/src/_core_clean.h"
 typedef struct flux_command flux_cmd_t;
 
 
@@ -1992,7 +1997,7 @@ typedef struct flux_command flux_cmd_t;
 
 
 typedef struct flux_subprocess flux_subprocess_t;
-# 3470 "/__w/flux-python/flux-python/src/_core_clean.h"
+# 3444 "/__w/flux-python/flux-python/src/_core_clean.h"
 typedef enum {
     FLUX_SUBPROCESS_INIT,
     FLUX_SUBPROCESS_RUNNING,
@@ -2065,9 +2070,9 @@ typedef void (*subprocess_log_f) (void *arg,
                                   int level,
                                   const char *fmt,
                                   va_list args);
-# 3554 "/__w/flux-python/flux-python/src/_core_clean.h"
+# 3528 "/__w/flux-python/flux-python/src/_core_clean.h"
 void flux_standard_output (flux_subprocess_t *p, const char *stream);
-# 3563 "/__w/flux-python/flux-python/src/_core_clean.h"
+# 3537 "/__w/flux-python/flux-python/src/_core_clean.h"
 flux_cmd_t * flux_cmd_create (int argc, char *argv[], char **env);
 
 
@@ -2142,12 +2147,12 @@ const char *flux_cmd_getenv (const flux_cmd_t *cmd, const char *name);
 
 int flux_cmd_setcwd (flux_cmd_t *cmd, const char *cwd);
 const char *flux_cmd_getcwd (const flux_cmd_t *cmd);
-# 3650 "/__w/flux-python/flux-python/src/_core_clean.h"
+# 3624 "/__w/flux-python/flux-python/src/_core_clean.h"
 int flux_cmd_add_channel (flux_cmd_t *cmd, const char *name);
-# 3700 "/__w/flux-python/flux-python/src/_core_clean.h"
+# 3674 "/__w/flux-python/flux-python/src/_core_clean.h"
 int flux_cmd_setopt (flux_cmd_t *cmd, const char *var, const char *val);
 const char *flux_cmd_getopt (flux_cmd_t *cmd, const char *var);
-# 3723 "/__w/flux-python/flux-python/src/_core_clean.h"
+# 3697 "/__w/flux-python/flux-python/src/_core_clean.h"
 flux_subprocess_t *flux_local_exec (flux_reactor_t *r,
                                     int flags,
                                     const flux_cmd_t *cmd,
@@ -2173,7 +2178,7 @@ flux_subprocess_t *flux_rexec_ex (flux_t *h,
                                   const flux_subprocess_ops_t *ops,
                                   subprocess_log_f log_fn,
                                   void *log_data);
-# 3758 "/__w/flux-python/flux-python/src/_core_clean.h"
+# 3732 "/__w/flux-python/flux-python/src/_core_clean.h"
 int flux_subprocess_stream_start (flux_subprocess_t *p, const char *stream);
 int flux_subprocess_stream_stop (flux_subprocess_t *p, const char *stream);
 int flux_subprocess_stream_status (flux_subprocess_t *p, const char *stream);
@@ -2193,11 +2198,11 @@ int flux_subprocess_write (flux_subprocess_t *p, const char *stream,
 
 
 int flux_subprocess_close (flux_subprocess_t *p, const char *stream);
-# 3789 "/__w/flux-python/flux-python/src/_core_clean.h"
+# 3763 "/__w/flux-python/flux-python/src/_core_clean.h"
 const char *flux_subprocess_read (flux_subprocess_t *p,
                                   const char *stream,
                                   int len, int *lenp);
-# 3804 "/__w/flux-python/flux-python/src/_core_clean.h"
+# 3778 "/__w/flux-python/flux-python/src/_core_clean.h"
 const char *flux_subprocess_read_line (flux_subprocess_t *p,
                                        const char *stream,
                                        int *lenp);
@@ -2208,10 +2213,10 @@ const char *flux_subprocess_read_line (flux_subprocess_t *p,
 const char *flux_subprocess_read_trimmed_line (flux_subprocess_t *p,
                                                const char *stream,
                                                int *lenp);
-# 3824 "/__w/flux-python/flux-python/src/_core_clean.h"
+# 3798 "/__w/flux-python/flux-python/src/_core_clean.h"
 int flux_subprocess_read_stream_closed (flux_subprocess_t *p,
                                         const char *stream);
-# 3843 "/__w/flux-python/flux-python/src/_core_clean.h"
+# 3817 "/__w/flux-python/flux-python/src/_core_clean.h"
 const char *flux_subprocess_getline (flux_subprocess_t *p,
                                      const char *stream,
                                      int *lenp);
@@ -2231,7 +2236,7 @@ flux_future_t *flux_subprocess_kill (flux_subprocess_t *p, int signo);
 
 void flux_subprocess_ref (flux_subprocess_t *p);
 void flux_subprocess_unref (flux_subprocess_t *p);
-# 3870 "/__w/flux-python/flux-python/src/_core_clean.h"
+# 3844 "/__w/flux-python/flux-python/src/_core_clean.h"
 flux_subprocess_state_t flux_subprocess_state (flux_subprocess_t *p);
 
 
@@ -2293,7 +2298,7 @@ void *flux_subprocess_aux_get (flux_subprocess_t *p, const char *name);
 
 void flux_subprocess_channel_incref (flux_subprocess_t *p, const char *name);
 void flux_subprocess_channel_decref (flux_subprocess_t *p, const char *name);
-# 3944 "/__w/flux-python/flux-python/src/_core_clean.h"
+# 3918 "/__w/flux-python/flux-python/src/_core_clean.h"
 extern "Python" void message_handler_wrapper(flux_t *, flux_msg_handler_t *, const flux_msg_t *, void *);
 
 
